# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - FormField Macro

    @copyright: 2012 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin import wikiutil
from MoinSupport import *

Dependencies = ['pages']

escape = wikiutil.escape

# Macro functions.

def execute(macro, args):

    """
    Execute the 'macro' with the given 'args' to produce a form field element:

      * A field name
      * The WikiDict describing form fields

    The following optional named arguments are also supported:

      label=TEXT    The label employed by button-like fields

    The nature of each field is described by a WikiDict entry for the given
    field name.
    """

    request = macro.request
    fmt = macro.formatter
    page = fmt.page
    _ = request.getText

    # Interpret the arguments.
    # NOTE: The argument parsing should really be more powerful in order to
    # NOTE: support labels.

    try:
        parsed_args = args and wikiutil.parse_quoted_separated(args, name_value=False) or []
    except AttributeError:
        parsed_args = args.split(",")

    parsed_args = [arg for arg in parsed_args if arg]

    # Get special arguments.

    name = None
    dictpage = None
    label = None

    for arg in parsed_args:
        if arg.startswith("name="):
            name = arg[5:]

        elif arg.startswith("dict="):
            dictpage = arg[5:]

        elif arg.startswith("label="):
            label = arg[6:]

        elif name is None:
            name = arg

        elif dictpage is None:
            dictpage = arg

    if not name:
        return showError(_("No field name specified."), request)

    if not dictpage:
        return showError(_("No WikiDict specified."), request)

    # Get the WikiDict and the field's definition.

    wikidict = getWikiDict(dictpage, request)

    if not wikidict:
        return showError(_("WikiDict %s cannot be loaded.") % dictpage, request)

    try:
        field_definition = wikidict[name]
    except KeyError:
        return showError(_("No entry for %s in %s.") % (name, dictpage), request)

    field_args = {}

    for field_arg in field_definition.split():

        # Record the key-value details.

        try:
            argname, argvalue = field_arg.split("=", 1)
            field_args[argname] = argvalue

        # Single keywords are interpreted as type descriptions.

        except ValueError:
            if not field_args.has_key("type"):
                field_args["type"] = field_arg

    # Obtain any request parameters corresponding to the field.

    form = get_form(request)
    value = form.get(name, [""])[0]

    # Render the field.

    type = field_args.get("type", "text")

    if type == "text":
        return fmt.rawHTML('<input name="%s" type="%s" size="%s" value="%s" />' % (
            escattr(name), escattr(type), escattr(field_args.get("size", "10")), escattr(value)
            ))

    elif type == "textarea":
        return fmt.rawHTML('<textarea name="%s" cols="%s" rows="%s">%s</textarea>' % (
            escattr(name), escattr(field_args.get("cols", "60")), escattr(field_args.get("rows", "5")), escape(value)
            ))

    elif type == "submit":
        return fmt.rawHTML('<input name="%s" type="submit" value="%s" />' % (
            escattr(name), escattr(_(label or name))
            ))

    # NOTE: Support select fields at the very least.

    else:
        return ''

def showError(text, request):
    fmt = request.formatter

    output = []
    append = output.append

    append(fmt.span(on=1, attrs={"class" : "form-field-error"}))
    append(fmt.text(text))
    append(fmt.span(on=0))

    return "".join(output)

# vim: tabstop=4 expandtab shiftwidth=4
